/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, see the file COPYING; if not, write
    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/

// This example illustrates the use of the range utility

#include "CM_util.h"

using namespace CheapMatrix;

main()
{
	// range in itself is a simple utility that returns a vector
	// from one point to another, equally spaced (default is 1)
	Vector r = range(5, 12);
	cout << r << endl;

	// You can also feed it negative increments
	Vector v = range(8.4, -0.7, 2.1);
	cout << v << endl;

	// In any case, it will not go beyond the limits
	v = range(0.2, 0.5, 1.0);
	cout << v << endl;

	// Are you missing Octave a(:, 1:2:7) ?
	// range can be very useful in that case
	Matrix a(2,8);
	a = 1, 4, 7, 2, 5, 8, 3, 6,
	    9, 1, 5, 9, 4, 7, 8, 0;
	cout << a(all, range(1,2,7)) << endl;

	// Works as fine reversed
	cout << a(all, range(8,-2,1)) << endl;

	// Or whith repeated entries and any arbitrary order
	Vector indices(5); indices = 3, 6, 4, 3, 1;
	Matrix b = a(all, indices);
	cout << b << endl;
	
	// But if you repeat, you have to be careful...
	b(1,1) = 3;
	cout << b << endl;
	
	// Simple solution is to give it its own memory buffer
	b.detach();
	b(1,1) = 0;
	cout << b << endl;

	// Also, all entries are converted to integers (rounded down)
	Vector ind2(3); ind2 = 1.1, 2.3, 4.9;
	cout << b(all, ind2) << endl;
}
