/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#ifndef EVD_H_CHEAP_MATRIX
#define EVD_H_CHEAP_MATRIX

#include "CheapMatrix.h"

// Don't pollute the standard namespace with our definitions
namespace CheapMatrix {

// Eigen Value Decomposition. Return the eigenvalues (real, imaginary)
// And the eigenvectors as columns of L, R
// Complex eigenvectors are 2 consecutive entries (re,im)
bool EVD(const Matrix &A, const Vector& reValues,
	const Vector& imValues, const Matrix& L, const Matrix& R);
bool EVD(const Matrix &A, const Vector& reValues,
	const Vector& imValues, const Matrix& R);
bool EVD(const Matrix &A, const Vector& reValues,
	const Matrix& L, const Matrix& R);
bool EVD(const Matrix &A, const Vector& reValues,
	const Matrix& R);
bool EVD(const Matrix &A, const Matrix& L, const Matrix& R);
bool EVD(const Matrix &A, const Matrix& R);
bool EVD(const Matrix &A, const Vector& reValues,
	const Vector& imValues);
bool EVD(const Matrix &A, const Vector& reValues);
// cannot overload with the same signature
bool left_EVD(const Matrix &A, const Vector& reValues,
	const Vector& imValues, const Matrix& L);
bool left_EVD(const Matrix &A, const Vector& reValues,
	const Matrix& L);
bool left_EVD(const Matrix &A, const Matrix& L);

// Same thing, optimised assuming the matrix A is symmetric
bool symmetric_EVD(const Matrix &A, const Vector& eigenValues,
	const Matrix& eigenVectors);
bool symmetric_EVD(const Matrix &A, const Vector& eigenValues);
bool symmetric_EVD(const Matrix &A, const Matrix& eigenVectors);

}

#endif


