/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef CM_ANNEALING_H
#define CM_ANNEALING_H

#include "CM_optimizer.h"

namespace CheapMatrix {

// Simulated annealing optimization algorithm
// Options:
// 1. max iter. Default is 100
// 2. initial temperature. Default is to try to find one.
// 3. ratio for updating temperature when a step is taken. Default 0.999
// 4. Step size standard deviation at initial temperature.
//    Default is 1 / total dimension
// 5. 'Boldzman' constant. Default is 1.0
// Auto-save will save the minimal error configuration, not necessarily
// the last step taken
class Annealing : public Optimizer
{
protected:
	MatrixType algorithm(OptimizeTarget& opta, const Matrix& data, const Matrix& target, const Vector& options);
public:
	inline Annealing(const Vector& options) : Optimizer(options) {}
	inline Annealing() {}
	Vector default_options();
};

}
#endif
