#
#    This file is part of the Cheap Matrix library
#    Copyright (C) 2000  Nicolas Brodu
#    nicolas.brodu@free.fr
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program, see the file COPYING; if not, write
#    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#    MA 02139, USA.

open(SOURCE, ">CheapMatrixOps.source");
open(HEADER, ">CheapMatrixOps.header");

print HEADER "// This file was generated by a perl script. Do not edit.\n\n";
#print HEADER "namespace CheapMatrix {\n\n";
print SOURCE "// This file was generated by a perl script. Do not edit.\n\n";
#print SOURCE "namespace CheapMatrix {\n\n";

@ops = ("+", "-");
@scalar_only_ops = ("*", "/");
@unary_funcs = ("tan", "sin", "cos", "exp", "log", "log10", "sqrt",
"acos", "asin", "atan", "cosh", "sinh", "tanh", "acosh", "asinh", "atanh",
"ceil", "fabs", "floor");
@binary_funcs = ("atan2", "pow", "fmod");

print HEADER "// Binary operators.\n";
print SOURCE "// Binary operators.\n";
foreach $op (@ops) {
print HEADER "::CheapMatrix::Matrix operator".$op."(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::Matrix& b);\n";
print HEADER "::CheapMatrix::Matrix operator".$op."=(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::Matrix& b);\n";
print SOURCE <<END_BINARY_OP_SOURCE
::CheapMatrix::Matrix operator$op(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::Matrix& b) {
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return ::CheapMatrix::Matrix();
	::CheapMatrix::Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res(i,j) = a(i,j) $op b(i,j);
		}
	}
	return res;
}
::CheapMatrix::Matrix operator$op=(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::Matrix& b) {
	int m = a.rows();
	int n = a.cols();
	assert(m==b.rows() && n==b.cols());
	if (!a || !b) return ::CheapMatrix::Matrix();
	if (!a.equal_ref(b)) {
		for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
			a(i,j) $op= b(i,j);
	} else {
		// possible overlap, use a temporary buffer
		::CheapMatrix::Matrix tmp(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
		for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
			tmp(i,j) = a(i,j) $op b(i,j);
		for (int j=1; j<=n; j++) for (int i=1; i<=m; i++)
			a(i,j) = tmp(i,j);
	}
	return a;
}
END_BINARY_OP_SOURCE
}
foreach $op (@ops,@scalar_only_ops) {
print HEADER "::CheapMatrix::Matrix operator".$op."(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::MatrixType& b);\n";
print HEADER "::CheapMatrix::Matrix operator".$op."(const ::CheapMatrix::MatrixType& a, const ::CheapMatrix::Matrix& b);\n";
print HEADER "::CheapMatrix::Matrix operator".$op."=(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::MatrixType& b);\n";
print SOURCE <<END_BINARY_OP_SCALAR_SOURCE
::CheapMatrix::Matrix operator$op(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::MatrixType& b) {
	if (!a) return ::CheapMatrix::Matrix();
	int m = a.rows();
	int n = a.cols();
	::CheapMatrix::Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res(i,j) = a(i,j) $op b;
		}
	}
	return res;
}
::CheapMatrix::Matrix operator$op(const ::CheapMatrix::MatrixType& a, const ::CheapMatrix::Matrix& b) {
	if (!b) return ::CheapMatrix::Matrix();
	int m = b.rows();
	int n = b.cols();
	::CheapMatrix::Matrix res(m, n, _temp_size, _temp_size?tmpnam(0):"", true);
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			res(i,j) = a $op b(i,j);
		}
	}
	return res;
}
::CheapMatrix::Matrix operator$op=(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::MatrixType& b) {
	if (!a) return ::CheapMatrix::Matrix();
	int m = a.rows();
	int n = a.cols();
	for (int j=1; j<=n; j++) {
		for (int i=1; i<=m; i++) {
			a(i,j) $op= b;
		}
	}
	return a;
}
END_BINARY_OP_SCALAR_SOURCE
}
print HEADER "\n";
print SOURCE "\n";

print HEADER "// Unary functions.\n";
foreach $func (@unary_funcs) {
print HEADER "inline ::CheapMatrix::Matrix ".$func."(const ::CheapMatrix::Matrix& a) {return map(::$func,a);}\n";
}
print HEADER "\n";

print HEADER "// Binary functions.\n";
foreach $func (@binary_funcs) {
print HEADER "inline ::CheapMatrix::Matrix ".$func."(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::Matrix& b) {return map(::$func,a,b);}\n";
print HEADER "inline ::CheapMatrix::Matrix ".$func."(const ::CheapMatrix::Matrix& a, const ::CheapMatrix::MatrixType& b) {return map(::$func,a,b);}\n";
print HEADER "inline ::CheapMatrix::Matrix ".$func."(const ::CheapMatrix::MatrixType& a, const ::CheapMatrix::Matrix& b) {return map(::$func,a,b);}\n";
}
print HEADER "\n";

#print HEADER "} // End of namespace\n";
#print SOURCE "} // End of namespace\n";

close HEADER;
close SOURCE;
